#!/bin/sh
PATH=/bin:/usr/bin
export PATH

PROG=$(basename $0)

AFLG=0
HFLG=0
FOPT=''
OOPT=''
OTIME=Y
ERR=0
while :
do
	case "$1" in
	-a)
		AFLG=1
		;;
	-h|-d)	# -d for old usage
		HFLG=1
		DIV=1024
		;;
	-H)
		HFLG=1
		DIV=1000
		;;
	-f)
		FOPT='-type f'
		;;
	-o*)
		OOPT="$OOPT "$(echo "$1" | sed 's/^-o//')
		;;
	-o)
		OOPT="$OOPT $2"
		shift
		;;
	-ta)
		OTIME=X
		;;
	-tb)
		OTIME=W
		;;
	-tc)
		OTIME=Z
		;;
	-tm)
		OTIME=Y
		;;
	-t?)
		ERR=1
		;;
	-t)
		case "$2" in
		a)
			OTIME=X
			;;
		b)
			OTIME=W
			;;
		c)
			OTIME=Z
			;;
		m)
			OTIME=Y
			;;
		*)
			ERR=1
			;;
		esac
		shift
		;;
	--)
		break
		;;
	-*)
		ERR=1
		break
		;;
	*)
		break
		;;
	esac
	shift
done

if [ "$ERR" = 1 ]
then
	echo "Usage: $PROG [-t{a|m|c}] [-f] [-h|H] [-a] [-o'<find options'>]... PATH..." >&2
	echo "	-ta	if newest/oldest, review access time" >&2
	echo "	-tb	if newest/oldest, review birth time" >&2
	echo "	-tc	if newest/oldest, review change time" >&2
	echo "	-tm	if newest/oldest, review modify time (default)" >&2
	echo "	-f	show only files" >&2
	echo "	-h	human readable form for dates and sizes (-h base 1024 / -H base 1000)" >&2
	echo "	-a	show all files (sorted)" >&2
	echo "	-o	Other options to find command" >&2
	exit 1
fi

case "$PROG" in
oldest*|newest*)
	OFF="%$OTIME"
	;;
biggest*|smallest*)
	OFF="%s"
	;;
esac

# find timestamps of given args
find "$@" $FOPT $OOPT -print0 | xargs -r -0 stat -c "$OFF %n" | sort -n | if [ "$AFLG" = 1 ]
then
	case "$PROG" in
	oldest*|smallest*)
		tac
		;;
	newest*|biggest*)
		cat
		;;
	esac
else
	case "$PROG" in
	oldest*|smallest*)
		sed 1q
		;;
	newest*|biggest*)
		tail -1
		;;
	esac
fi | if [ "$HFLG" != 0 ]
then
	case "$PROG" in
	oldest*|newest*)
		perl -n -e '($t, $a)=split(" ", $_, 2);@l=localtime($t);printf "%04d/%02d/%02d-%02d:%02d:%02d %s", $l[5]+1900, $l[4]+1, $l[3], $l[2], $l[1], $l[0], $a'
		;;
	biggest*|smallest*)
		perl -n -e '($s, $a)=split(" ", $_, 2);
			if ($s < '"$DIV"') {
				printf "%d %s", $s, $a;
			} elsif ($s < '"$DIV*$DIV"') {
				printf "%.1fK %s", $s'"/$DIV"', $a;
			} elsif ($s < '"$DIV*$DIV*$DIV"') {
				printf "%.1fM %s", $s'"/$DIV/$DIV"', $a;
			} elsif ($s < '"$DIV*$DIV*$DIV*$DIV"') {
				printf "%.1fG %s", $s'"/$DIV/$DIV/$DIV"', $a;
			} elsif ($s < '"$DIV*$DIV*$DIV*$DIV*$DIV"') {
				printf "%.1fT %s", $s'"/$DIV/$DIV/$DIV/$DIV"', $a;
			} else {
				printf "%.1fP %s", $s'"/$DIV/$DIV/$DIV/$DIV/$DIV"', $a;
			}
			'
		;;
	esac
else
	cat
fi
